#ifndef BDFILE_H
#define BDFILE_H

/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <string>

#include "u2d_core.h"

#define FORMAT_UMESH2D 1
#define FORMAT_UMESH2D_FE 2
#define FORMAT_UMESH2D_FV 3

  /* Scheduled for removal */
#define MAX_FILE_LENGTH 512

using namespace std;

class BDFile {
public:
  BDFile();
  ~BDFile();

  /* PRIMITIVE LIST SELECTION */
  void select_prim_list(Primitive *first, Primitive *last, int prim_number, int vertex_n);

  /* DOMAIN FILE SELECTION */
  void set_steiners(steiner *steiners_, int number);
  void set_backgrids(backgrid *backgrid_, int number);

  /* OUTPUT TYPE SELECTION */
  void set_output_format(int format);

  /* FILE GENERATION */

  /* FILE OUTPUT */
  void set_project_name(const char *m_project_name);
  const char *get_project_name();
  int outputbfile();
  int outputdfile();
  int output();

private:
  /* OUTPUT TYPE SELECTION */
  int output_format;
  /* DOMAIN FILE INFORMATION */
  steiner *steiners;
  backgrid *backgrids;
  int steiner_n;
  int backgrid_n;

  /* PRIMITIVE INFORMATION */
  Primitive *firstprimitive;
  Primitive *lastprimitive;
  int elements; // number of primitives in the list, length of the list.
  int vertexs; // no idea what is this!

  /* FILE GENERATION */
  inline void printhash(FILE *output);
  inline void printformat(FILE *output, int format);
  inline void printgeom(FILE *output);
  inline void printtopo(FILE *output);
  inline void printnvne(FILE *output);
  void printverelem(FILE *output);
  inline void printish(FILE *output);
  inline void printenbvev(FILE *output);
  void printprimg(FILE *output, Primitive *curprim);
  void printprimd(FILE *outptu, Primitive *curprim);
  void printbegin(FILE *output, int id);
  void printbegin(FILE *output, int id, char *name);
  inline void printend(FILE *output);
  void printpriminfo(FILE *output);
  void printprimdata(FILE *output);

  /* FILE OUTPUT */
  int openbfile();
  int opendfile();
  string project_name;
  string boundary_filename; //Must allready contain rigth boundary.name
  string domain_filename; //Must allready contain rigth domain.name
  void set_boundary_filename();
  void set_domain_filename();
  FILE *bfile;
  FILE *dfile;


  /* Scheduled for removal */
  void addboundary(char *buffer);
  /* Scheduled for removal */
  void adddomain(char *buffer);
};

#endif  /* BDFILE_H */
