#ifndef U2D_GLMESH_H
#define U2D_GLMESH_H

/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
    Copyright (C) 2004 Gabriele Dini Ciacci
    Copyright (C) 2005 Gabriele Dini Ciacci
    Copyright (C) 2006 Gabriele Dini Ciacci
    Copyright (C) 2007 Gabriele Dini Ciacci
    Copyright (C) 2008 Gabriele Dini Ciacci

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <fltk/gl.h>
#include <fltk/GlWindow.h>
#include <stdio.h>
#include <GL/gl.h>
#include <vector>

#include <u2d_display_set.h>

using namespace std;

class u2d_GLmesh : public fltk::GlWindow {
  void draw();
  int handle(int event);

 public:
  u2d_GLmesh(int X, int Y, int W, int H, const char *L) : fltk::GlWindow(X, Y, W, H, L) {
    ref_minX = 0;
    ref_maxX = 0;
    ref_minY = 0;
    ref_maxY = 0;
    real_minX = 0;
    real_maxX = 0;
    real_minY = 0;
    real_maxY = 0;
    last_push_x = 0;
    last_push_y = 0;
    current_factor = 1;
    set_ref_sizes(-2, 2, -2, 2);
  }
  /* For drawing reference size */
  void get_ref_sizes(double *minX, double *maxX, double *minY, double *maxY);
  void guess_ref_size(); // <--- undefined for now
  void get_real_sizes(double *minX, double *maxX, double *minY, double *maxY);

  /* Drawing */
  GLuint create_display_list();
  void load_all_display_set();
  void call_all_display_set();
  /* still to write */
  void set_vertex_array(void *array, unsigned int length);
  void load_vertex_array();

  int load_display_set(u2d_Display_Set *display_set);
  void pan(double disp_x, double disp_y);
  void zoom(double delta_factor, int click_X, int click_Y);
  void centered_zoom(double delta_factor);
/* TODO: Make the next one private and add fuctions to access it. */
  vector<u2d_Display_Set> display_set;
/* TODO: add functions to remove a display set */

 private:
  void MyInit();
  void set_default_colors();
  void set_default_antialiasing();
  /* For drawing reference size, zoom and pan */
  void set_ref_sizes(double minX, double maxX, double minY, double maxY);
  int last_push_x;
  int last_push_y;
  double current_factor;
  double ref_minX;
  double ref_maxX;
  double ref_minY;
  double ref_maxY;
  double real_minX;
  double real_maxX;
  double real_minY;
  double real_maxY;
};

#endif
